#include <iostream>
#include <math.h>
#include <csignal>
#include <fstream>
#include "KINECTRecorder.hpp"


#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

using namespace std;
using namespace boost;
using namespace boost::posix_time;


scoped_ptr<KINECTRecorder> KINECTRecorder::instance;
//***************************************************************************************//
// This function is executed in another thread, and
// records acoustic signals into circular buffer.
//***************************************************************************************//
void KINECTRecorder::operator()()
{
//	read_buf.resize(channel_count * (header_period - 1) * 20);
	read_buf.resize(channel_count * (header_period) * 20);
	vector<char> buf(sample_bytes * channel_count * header_period * 20);
	int read_size    = 0;
	int header_count = 0;

	ptime t_start = microsec_clock::local_time();
	ptime t_cur;

	cout << "KINECTRecorder::operator START" << endl;
	cout << "  channel_count: " << channel_count << endl;
	cout << "  header_period: " << header_period << endl;
	cout << "  sample_bytes:  "  << sample_bytes << endl;
	
	//ofstream fout;
	//fout.open("recv-data.txt", ios::out | ios::app | ios::binary);
	
	int ret;

	for (;;) 
	{
		if (state == PAUSE) 
		{
			cout << "RECORDING PAUSE" << endl;
			usleep(sleep_time);
			continue;
		}
		else if (state == STOP) 
		{
			cout << "RECORDING STOP" << endl;
			break;
		}
		else if (state == RECORDING) 
		{
			//cout << "KINECT RECORDING START" << endl;
			read_size = KcSampleDataRead(h_ws, (char*)&buf[0], buf.size());
			//cout << "read_size = " << read_size  << endl;

			//fout.write(&buf[0] ,read_size);


			if (read_size <= 0) 
			{
				break;
			}

			int frame_count = read_size / channel_count / sample_bytes;

			int i = 0;
			int j = 0;

			while ( j < frame_count ) 
			{

				int start_rb  = i * channel_count;
				int start_buf = sample_bytes * j * channel_count;
				int end_buf   = start_buf + sample_bytes * channel_count;

				switch (sample_bytes) 
				{
					case 2: //16bit mode
					{
						cout << "16bit mode" << endl;
						int16_t* first = (int16_t*)&buf[start_buf];
						int16_t* last  = (int16_t*)&buf[end_buf];
						copy(first, last, read_buf.begin() + start_rb);
						break;
					}
					case 4: //32bit mode
					{
						int32_t* first = (int32_t*)&buf[start_buf];
						int32_t* last  = (int32_t*)&buf[end_buf];
//							float* first = (float*)&buf[start_buf];
//							float* last  = (float*)&buf[end_buf];


						copy(first, last, read_buf.begin() + start_rb);
						break;
					}
				}
			
//				cout << "before  cur_time increment. cur_time=" << cur_time << endl;
				cur_time++;
//				cout << "after cur_time increment. cur_time=" << cur_time << endl;

				i++;
				j++;

			}//while end


			t_cur = microsec_clock::local_time();
//			cout << "sample_time_queue.push_back start" << endl;
			sample_time_queue.push_back(sample_time(cur_time, t_cur - microseconds(51200)));
//			cout << "sample_time_queue.push_back end" << endl;
			int end_time = cur_time - buffer.size() / channel_count;
//			cout << "end_time" << end_time << endl;
			std::deque<sample_time>::iterator it_st;

			for (it_st = sample_time_queue.begin() ; it_st != sample_time_queue.end(); ++it_st) 
			{
				if (it_st->sample >= end_time) 
				{
					break;
				}
			}

//			cout << "sample_time_queue.erase start"  << endl;
			sample_time_queue.erase(sample_time_queue.begin(), it_st);
//			cout << "cur_time=" << cur_time << " " << "total_microseconds=" << (t_cur - t_start).total_microseconds() - 10500 - 51200 << endl;

			vector<float>::iterator it = read_buf.begin();
			advance(it, i * channel_count);
			mutex::scoped_lock lk(mutex_buffer);
//			cout << "buffer.insert start i->"  << i << " buffer.size->" << buffer.size() << endl;
			buffer.insert(buffer.end(), read_buf.begin(), it);
//			cout << "sleep_time1->" << sleep_time << endl;
			ret= usleep(sleep_time);
//			cout << "ret ->" << ret << " sleep2 end buffer.size()->" << buffer.size() << endl;
			lk.unlock();
		}//if end
	}
	
	//fout.close();
	cout << "KINECTRecorder::operator END" << endl;
}



//***************************************************************************************//
// A device for recording is initialized.
//   device_name : Audio device name.  ex. "plughw:0,0"
//   device_type : Audio device type.  ex. "RASP24-32 or RASP24-16"
//   chan_count  : A number of channels. ex. 8 if eight microphones are used.
//   samp_rate   : A frequency of sampling.
//   buf_size    : A size of circular buffer.
//***************************************************************************************//
int KINECTRecorder::Initialize( const string& device_name,
                                int    chan_count,
                                int    samp_rate,
                                size_t buf_size )
{
	
	mutex::scoped_lock lk(mutex_buffer);

	recorderThread.reset();
	buffer.clear();
	sample_time_queue.clear();

	//read_buf_size=buf_size;
	state         = PAUSE;
	buffer_size   = buf_size;
	channel_count = chan_count;
	sampling_rate = samp_rate;
	buffer.set_capacity(buffer_size * channel_count);
	read_buf.resize(read_buf_size * channel_count);
	cur_time = 0;
	sleep_time = 160 * 1000000 / sampling_rate;

	header_period = 2560;
	sample_bytes = 4;

	cout << "KINECTRecorder::Initialize START"  << endl;
	cout << "buffer_size="   << buf_size << endl;
	cout << "channel_count=" << chan_count << endl;
	cout << "sampling_rate=" << samp_rate << endl;
	cout << "sample_bytes="  << sample_bytes << endl;
	cout << "read_buf="      << read_buf.size() << endl;
	cout << "header_period="      << header_period << endl;


	/*-----------------------------------------------------------*/
	/* Handle Close                                              */
	/*-----------------------------------------------------------*/

	if (h_ws) 
	{
		
		cout << "KcHandleClose START"  << endl;
		KcHandleClose(h_ws);
	}


	/*-----------------------------------------------------------*/
	/* Handle Open                                               */
	/*-----------------------------------------------------------*/
//	cout << "KcHandleOpen START"  << endl;
	h_ws = KcHandleOpen((char*)device_name.c_str());

	if (!h_ws) 
	{
		cerr << "Cannot open Kinect.\n";
		return -1;
	}

	cout << "KcHandleOpen END" << endl;



	/*-----------------------------------------------------------*/
	/* 終了処理                                                  */
	/*-----------------------------------------------------------*/
//	KcADStop(h_ws);
	recorderThread.reset(new thread(ref(*this)));
	return 0;
}



