/**
Copywright (c)2006 Nittobo Acoustic Engineering,Co.,Ltd.

\file MICARY_GHDSS_struct.h
\brief\~Japanese GHDSSp\
\author Sachiko Suzuki
\date 5 Oct 2006
\Version $Id: MICARY_GHDSS_struct.h 3538 2012-02-29 07:25:35Z tanaka $
*/
// ---------------------------------------------------------------------
#ifndef	MICARY_GHDSS_struct_h_defined
#define	MICARY_GHDSS_struct_h_defined
// ---------------------------------------------------------------------
#include	<stdio.h>
#include	"MICARY_def.h"
#include	"MICARY_struct.h"
#include	"ArrayInit.h"
// ---------------------------------------------------------------------
// 2007/07/09 added by S.Suzuki C++CĂяo
#ifdef __cplusplus
extern "C" {
#endif

// CmdLinesArgs̏

// --------------------------------------------------------------------
/**
The enum contains Command TF Referense form .
\author Sachiko Suzuki
\date 5 Oct 2006
*/

typedef enum{
	ntrCalcFree,
	ntrDB
} MICARY_TFRefEnum ;

/**
The enum contains Command separation matrix initialize form .
\author Sachiko Suzuki
\date 8 Dec 2006
*/

typedef enum{
	nsmInit,
	nsmLib
} MICARY_SepMatInitEnum ;

//----------------------------------------------------------------------

/**
The structure contains Command Line Args.
\author Sachiko Suzuki
\date 5 Oct 2006
*/
typedef struct {
	HA_String cOriginalWavFile ;	///<\~Japanese ̓t@C
	HA_Float ha_fStartTime ;	///<\~Japanese Jn[sec]
	HA_Float ha_fEndTime ;		///<\~Japanese I[sec]
	HA_String cSeparateWavFile ;	///<\~Japanese t@C
	HA_String cOriginalSpecFile ;	///<\~Japanese ̓XyNgt@C
	HA_Integer ha_iStartFrame ;	///<\~Japanese ͊Jnt[[frame]
	HA_Integer ha_iEndFrame ;	///<\~Japanese ͏It[[frame]
	HA_String cSeparateSpecFile ;	///<\~Japanese o̓XyNgt@C
	HA_String cSrcLocFile ;			///<\~Japanese ʒut@C
	HA_String dMicLocFile ;			///<\~Japanese 󉹓_ʒut@C
	HA_String cTFDBFile ;			///<\~Japanese TFf[^x[X
	MICARY_TFRefEnum ntrTFRef ;	///<\~Japanese TFQƕ@
	HA_Integer ha_iNFFT ;		///<\~Japanese FFT[point]
	HA_Integer ha_iWindow ;		///<\~Japanese [sample]
	MICARY_WindowEnum micary_nwnWindow ;	///<\~Japanese 
	HA_Integer ha_iShift ;		///<\~Japanese Vtg[sample]
	HA_Float ha_fUpperFreq ;	///<\~Japanese ͏g[Hz]
	HA_Float ha_fStepSizePrm ;	///<\~Japanese XebvTCYp[^
	HA_Float ha_fTemperature ;	///<\~Japanese C[]
	MICARY_SepMatInitEnum  ntrSepMat ;	///<\~Japanese s̏ 
	HA_Float ha_fOutMag ;           ///<\~Japanese o̓QC 

} MICARY_GHDSSCmdArgs ;

// --------------------------------------------------------------------
/**
The structure contains Separation matrix.
\author Sachiko Suzuki
\date 5 Oct 2006
*/
typedef struct{
	MICARY_IntArray micary_iaFreqLinex ;	///<\~Japanese gCfbNX
	HA_Num ha_numInput ;			///<\~Japanese ͐M
	HA_Num ha_numOutput ;			///<\~Japanese o͐M
	MICARY_ComplexArray micary_cmpaSeparation ;	///<\~Japanese s
	// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num ha_numReservedInput;	///<\~ ͐M̊mې
	HA_Num ha_numReservedOutput;	///<\~ o͐M̊mې
#endif
} MICARY_SeparationMat ;

//----------------------------------------------------------------------
/**
The structure contains Saved Separation.
\author Sachiko Suzuki
\date 5 Oct 2006
*/
typedef struct{
	MICARY_TFDatabase micary_tdTFDB ;			///<\~Japanese TFǂݏoۑl
	MICARY_SeparationMat micary_smSeparation ;	///<\~Japanese s
} MICARY_SavedSeparation ;

// ---------------------------------------------------------------------
/**
The structure contains Parameter Separation.
\author Sachiko Suzuki
\date 5 Oct 2006
*/
typedef struct{
	HA_Float ha_fStepSizePrm ;	///<\~Japanese XebvTCYp[^
} MICARY_PrmSeparation ;

//-----------------------------------------------------------------------
/**
The structure contains Parameter Separation for ASOCRA.
\author Sachiko Suzuki
\date 14 May 2008
*/
typedef struct{
	HA_Integer	ha_iSSMethod ; ///<\~Japanese SS@
	HA_Float ha_fSSMyu ;	///<\~Japanese SSpXebvTCYp[^
	HA_Integer	ha_iLCMethod ;	///<~Japanese LC@
	HA_Float	ha_fLCMyu ;	///<~Japanese	LCpXebvTCYp[^
	HA_Integer	ha_iAvg_Method ;	 ///<~Japanese	ω@
	HA_Float	ha_fAlpha ;	///<~Japanese ω̏d݌W
	HA_Float	ha_fBeta ;	///<~Japanese W
	HA_Integer	ha_iNmax ;	 ///<~Japanese ̍ől
	HA_Float	ha_fTran ;	///<~Japanese trancation
} MICARY_PrmSeparationASOCRA ;

//-----------------------------------------------------------------------

// 2008/09/03 changed by N.Tanaka
// VersionԍCt@C^Cv̒ǉ
/*
/ **
The structure contains TFDataBase File header.
\author Sachiko Suzuki
\date 5 Oct 2006
* /
typedef struct{
	int lFs ;		///<\~Japanese TvOg[Hz]
	int lNFFT ;	///<\~Japanese FFT[point]
	int lNMic ;	///<\~Japanese }CNzʒu
	int lNSrc ;	///<\~Japanese ʒu
	int lNFreq ;	///<\~Japanese gC[line]
	int lMeshFlg ;	///<\~Japanese ʒuiqɂȂĂ邩flag
	int lMeshX ;	///<\~Japanese X̊iq
	int lMeshY ;	///<\~Japanese Y̊iq
	int lMeshZ ;	///<\~Japanese Z̊iq
	char cPad[100] ;	///<\~Japanese pbh
} MICARY_TFDBFileHeader ;
*/
/**
The structure contains TFDataBase File header.
\author Sachiko Suzuki
\date 5 Oct 2006
*/
typedef struct{
	int lFs ;		///<\~Japanese TvOg[Hz]
	int lNFFT ;	///<\~Japanese FFT[point]
	int lNMic ;	///<\~Japanese }CNzʒu
	int lNSrc ;	///<\~Japanese ʒu
	int lNFreq ;	///<\~Japanese gC[line]
	int lMeshFlg ;	///<\~Japanese ʒuiqɂȂĂ邩flag
	int lMeshX ;	///<\~Japanese X̊iq
	int lMeshY ;	///<\~Japanese Y̊iq
	int lMeshZ ;	///<\~Japanese Z̊iq
	int iVersion;	///<\~Japanese o[Wԍ
	int iFileType;	///<\~Japanese t@C^Cvi0:TF_CONJ, 1:sj
	char cPad[92] ;	///<\~Japanese pbh
} MICARY_TFDBFileHeader ;


/**
 2Dimention array struct
 \author	Sachiko Suzuki
 \date	27 Nov 2006
*/
typedef struct{
	HA_Float **	ha_fValue; ///<\~Japanese HA_Floatz
	HA_Num	ha_numValue1; ///<\~Japanese Pz̃TCY
	HA_Num ha_numValue2 ; ///<\~Japanese Qz̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved1; ///<\~Japanese z̎ۂ̃TCY
	HA_Num	ha_numReserved2; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_FloatArray2;

/**
 2Dimention complex array struct
 \author	Sachiko Suzuki
 \date	27 Nov 2006
*/
typedef struct{
	HA_Complex **	ha_cmpValue; ///<\~Japanese HA_Complexz
	HA_Num	ha_numValue1; ///<\~Japanese Pz̃TCY
	HA_Num ha_numValue2 ; ///<\~Japanese Qz̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved1; ///<\~Japanese z̎ۂ̃TCY
	HA_Num	ha_numReserved2; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_ComplexArray2;

/**
 GHDSS_Location Array struct
 \author	Sachiko Suzuki
 \date	19 Dec 2006
*/
typedef struct{
	HA_Id *	ha_idLocation; ///<\~Japanese ʒuWIDz
	HA_Id *	ha_idExistence; ///<\~Japanese ʒuWz
	int micary_MoveFlg ;	///<\~Japanese ړtO
	HA_Num ha_numLocation ;	///<\~Japanese Wʒu
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_GHDSSLocationId;

/**
 GHDSS_Frequency Array struct
 \author	Sachiko Suzuki
 \date	26 Feb 2007
*/
typedef struct{
	MICARY_IntArray *micary_iaFreqLine ;	///<\~Japanese gCz
	HA_Integer 	ha_iLowerFreq; ///<\~Japanese gCԍ
	HA_Integer 	ha_iUpperFreq; ///<\~Japanese gCԍ
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_GHDSSFreqLine;


// ---------------------------------------------------------------------
/**
The structure contains Parameter Separation.
\author Sachiko Suzuki
\date 12 May 2008
*/
typedef struct{
	HA_Integer	ha_iSSMethod ;	///<\~Japanese SSXebvTCYƍXVʂ̌vZ@(0:Fix,1:myu_LC,2:Adaptive)
	HA_Float	ha_fSSMyu ;	///<\~Japanese SSpXebvTCYp[^
	HA_Integer	ha_iOCRAMethod ;	///<\~Japanese OCRA@(0:No,1:alpha->Fix,2:alpha->Adaptive)
	HA_Float	ha_fAlpha ;	///<\~Japanese ω̏d݌W
	HA_Float	ha_fBeta ; ///<\~Japanese ̍ۂ̌W
	HA_Integer	ha_iNmax ;	///<\~Japanese ̍ől
	HA_Float	ha_fNoiseFloor ;	///<\~Japanese mCYtAl
	HA_Integer	ha_iLCConst ;	///<\~Japanese LC_Const@(0:Full ,1:Diag)
	HA_Integer	ha_iLCMethod ;	///<\~Japanese LCXebvTCYƍXVʂ̌vZ@(0:Fix,1:Adaptive)
	HA_Float	ha_fLCMyu ;	///<\~Japanese LCpXebvTCYp[^
} MICARY_PrmSeparation_ASOCRA ;

//----------------------------------------------------------------------
/**
The structure contains Parameter Separation for GHDSS.
\author Sachiko Suzuki
\date 24 Dec 2009
*/
typedef struct{
	HA_Integer	ha_iSSMethod ;	///<\~Japanese SSXebvTCYƍXVʂ̌vZ@(0:Fix,1:myu_LC,2:Adaptive)
	HA_Float	ha_fSSMyu ;	///<\~Japanese SSpXebvTCYp[^
	HA_Float	ha_fSSScal ;	///<\~Japanese SSpXP[Ot@N^[
	HA_Float	ha_fNoiseFloor ;	///<\~Japanese mCYtAl
	HA_Integer	ha_iLCConst ;	///<\~Japanese LC_Const@(0:Full ,1:Diag)
	HA_Integer	ha_iLCMethod ;	///<\~Japanese LCXebvTCYƍXVʂ̌vZ@(0:Fix,1:Adaptive)
	HA_Float	ha_fLCMyu ;	///<\~Japanese LCpXebvTCYp[^
} MICARY_PrmSeparation_GHDSS ;

// ---------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////////////////////
/**
 \brief \~English It allocates float array(dim:2)
 \brief \~Japanese float^̓񎟌zmۂ
 \param array \~English It's the array of float. (iArraySize1 * iArraySize2)
 \~Japanese float^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
*/
void FloatArray2Alloc(MICARY_FloatArray2 * array, int iArraySize1, int iArraySize2) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates float array(dim:2)
 \brief \~Japanese float^̓񎟌zmۂ
 \param array \~English It's the array of float. (iArraySize1 * iArraySize2)
 \~Japanese float^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
 \param iReservedSize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔z̊mی
 \param iReservedSize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔z̊mی
*/
void FloatArray2Alloc_r(MICARY_FloatArray2 * array, int iArraySize1, int iArraySize2,
												  int iReservedSize1, int iReservedSize2) ;
#endif

/**
 \brief \~English It frees float array(dim:2)
 \brief \~Japanese float^̓񎟌zJ
 \param array \~English It's the array of float. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
*/
void FloatArray2Free(MICARY_FloatArray2 * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees float array(dim:2)
 \brief \~Japanese float^̓񎟌z̎gpTCYNA
 \param array \~English It's the array of float. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
*/
void FloatArray2Clear(MICARY_FloatArray2 * array);
#endif

/**
 \brief \~English It allocates complex array(dim:2)
 \brief \~Japanese complex^̓񎟌zmۂ
 \param array \~English It's the array of complex. (iArraySize1 * iArraySize2)
 \~Japanese complex^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
*/
void ComplexArray2Alloc(MICARY_ComplexArray2 * array, int iArraySize1, int iArraySize2) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates complex array(dim:2)
 \brief \~Japanese complex^̓񎟌zmۂ
 \param array \~English It's the array of complex. (iArraySize1 * iArraySize2)
 \~Japanese complex^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
 \param iReservedSize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔z̊mی
 \param iReservedSize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔z̊mی
*/
void ComplexArray2Alloc_r(MICARY_ComplexArray2 * array, int iArraySize1, int iArraySize2,
													  int iReservedSize1, int iReservedSize2) ;
#endif

/**
 \brief \~English It frees complex array(dim:2)
 \brief \~Japanese complex^̓񎟌zJ
 \param array \~English It's the array of complex. (iArraySize1 * iArraySize2)
 \~Japanese complex^̓񎟌z̃|C^
*/
void ComplexArray2Free(MICARY_ComplexArray2 * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees complex array(dim:2)
 \brief \~Japanese complex^̓񎟌z̎gpNA
 \param array \~English It's the array of complex. (iArraySize1 * iArraySize2)
 \~Japanese complex^̓񎟌z̃|C^
*/
void ComplexArray2Clear(MICARY_ComplexArray2 * array);
#endif

/**
 \brief \~English It allocates separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_SeparationMat\̂̊mہi1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese s\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of microphone .
 \~Japanese }CN
*/
void SeparationMatAlloc(MICARY_SeparationMat * SPM, int iFreqSize, int iLocSize, int iMicSize) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_SeparationMat\̂̊mہi1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese s\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of microphone .
 \~Japanese }CN
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese mۂvf(Freq)
 \param iReservedLocSize \~English It's the size of array
 \~Japanese mۂvf(Location)
 \param iReservedMicSize \~English It's the size of array
 \~Japanese mۂvf(Mic)
*/
void SeparationMatAlloc_r(MICARY_SeparationMat * SPM, int iFreqSize, int iLocSize, int iMicSize,
													int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize) ;
#endif

/**
 \brief \~English It frees separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_SeparationMat\̂Ji1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese s\̃|C^
*/
 void SeparationMatFree(MICARY_SeparationMat * SPM) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_SeparationMat\̂̎gpNAi1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese s\̃|C^
*/
 void SeparationMatClear(MICARY_SeparationMat * SPM) ;
#endif

/**
 \brief \~English It allocates saved separation .(1:true, 0:false)
 \brief \~Japanese MICARY_SavedSeparation\̂̊mہi1:IC0:G[j
 \param SSP \~English It's the saved separation struct pointer .
 \~Japanese ۑl\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of microphone .
 \~Japanese }CN
*/
void SavedSeparationAlloc(MICARY_SavedSeparation * SSP ,int iFreqSize, int iLocSize, int iMicSize) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates saved separation .(1:true, 0:false)
 \brief \~Japanese MICARY_SavedSeparation\̂̊mہi1:IC0:G[j
 \param SSP \~English It's the saved separation struct pointer .
 \~Japanese ۑl\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of microphone .
 \~Japanese }CN
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese mۂvf(Freq)
 \param iReservedLocSize \~English It's the size of array
 \~Japanese mۂvf(Location)
 \param iReservedMicSize \~English It's the size of array
 \~Japanese mۂvf(Mic)
*/
void SavedSeparationAlloc_r(MICARY_SavedSeparation * SSP ,int iFreqSize, int iLocSize, int iMicSize,
						  int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize) ;
#endif

/**
 \brief \~English It frees saved separation.(1:true, 0:false)
 \brief \~Japanese MICARY_SavedSeparation\̂Ji1:IC0:G[j
 \param SSP \~English It's the saved separation struct pointer .
 \~Japanese ۑl̃|C^
*/
void SavedSeparationFree(MICARY_SavedSeparation * SSP) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/*
 \brief \~English It frees saved separation.(1:true, 0:false)
 \brief \~Japanese MICARY_SavedSeparation\̂Ji1:IC0:G[j
 \param SSP \~English It's the saved separation struct pointer .
 \~Japanese ۑl̃|C^
*/
void SavedSeparationClear(MICARY_SavedSeparation * SSP) ;
#endif

/**
 \brief \~English It allocates GHDSS Location ID .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSLocationId\̂̊mہi1:IC0:G[j
 \param array \~English It's the location ID(for GHDSS) struct pointer .
 \~Japanese GHDSSWIDz\̃|C^
 \param iArraySize \~English It's the number of Location.
 \~Japanese ʒuW
*/
void LocationIdAlloc(MICARY_GHDSSLocationId * array, int iArraySize) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/*
 \brief \~English It allocates GHDSS Location ID .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSLocationId\̂̊mہi1:IC0:G[j
 \param array \~English It's the location ID(for GHDSS) struct pointer .
 \~Japanese GHDSSWIDz\̃|C^
 \param iArraySize \~English It's the number of Location.
 \~Japanese ʒuW
 \param iReservedSize \~English It's the number of Location.
 \~Japanese mۂʒuW
*/
void LocationIdAlloc_r(MICARY_GHDSSLocationId * array, int iArraySize, int iReservedSize) ;
#endif

/**
 \brief \~English It frees GHDSS Location ID .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSLocationId\̂Ji1:IC0:G[j
 \param array \~English It's the location ID(for GHDSS) struct pointer .
 \~Japanese GHDSSWIDz\̃|C^
 */
void LocationIdFree(MICARY_GHDSSLocationId * array) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees GHDSS Location ID .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSLocationId\̂̎gpNAi1:IC0:G[j
 \param array \~English It's the location ID(for GHDSS) struct pointer .
 \~Japanese GHDSSWIDz\̃|C^
 */
void LocationIdClear(MICARY_GHDSSLocationId * array) ;
#endif

/**
 \brief \~English It allocates GHDSS frequency line struct .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSgC\̂̊mہi1:IC0:G[j
 \param array \~English It's the frequency line struct pointer .
 \~Japanese gC\̃|C^
 \param iLowerFreqLine \~English It's the lower frequency line number .
 \~Japanese gCԍ
 \param iUpperFreqLine \~English It's the upper frequency line number .
 \~Japanese gCԍ
*/
void FreqLineAlloc(MICARY_GHDSSFreqLine * array, int iLowerFreqLine , int iUpperFreqLine) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates GHDSS frequency line struct .(1:true, 0:false)
 \brief \~Japanese MICARY_GHDSSgC\̂̊mہi1:IC0:G[j
 \param array \~English It's the frequency line struct pointer .
 \~Japanese gC\̃|C^
 \param iLowerFreqLine \~English It's the lower frequency line number .
 \~Japanese gCԍ
 \param iUpperFreqLine \~English It's the upper frequency line number .
 \~Japanese gCԍ
 \param iReservedSize \~English It's size of array .
 \~Japanese mۂvf
*/
void FreqLineAlloc_r(MICARY_GHDSSFreqLine * array, int iLowerFreqLine , int iUpperFreqLine, int iReservedSize) ;
#endif

/*
 \brief \~English It frees frequency line struct .(1:true, 0:false)
 \brief \~Japanese gC\̂Ji1:IC0:G[j
 \param array \~English It's the frequency line struct pointer .
 \~Japanese gC\̃|C^
 */
void FreqLineFree(MICARY_GHDSSFreqLine * array) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/*
 \brief \~English It frees frequency line struct .(1:true, 0:false)
 \brief \~Japanese gC\̂̎gpNAi1:IC0:G[j
 \param array \~English It's the frequency line struct pointer .
 \~Japanese gC\̃|C^
 */
void FreqLineClear(MICARY_GHDSSFreqLine * array) ;
#endif


/**
 Database of Separation matrix
 \author	Sachiko Suzuki
 \date	10 Jan 2008
*/

typedef struct{
	MICARY_SeparationMat micary_smSeparation ;	///<\~Japanese s	
	MICARY_LocationArray	micary_locaSource; ///<\~Japanese `B֐̉ʒuWz
	MICARY_LocationArray	micary_locaMic; ///<\~Japanese `B֐̃}CNzʒuWz
} MICARY_SMDatabase;

/**
 \brief \~English It allocates database of separation matrix
 \brief \~Japanese si[zmۂ
 \param smD \~English It's Database of separation matrix
 \~Japanese si[z܂ލ\
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese `B֐̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese `B֐̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese `B֐̃}CNzʒu
*/
void SMDatabaseAlloc(MICARY_SMDatabase * SMD, int iFreqSize, int iLocSize, int iMicSize) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates database of separation matrix
 \brief \~Japanese si[zmۂ
 \param smD \~English It's Database of separation matrix
 \~Japanese si[z܂ލ\
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese `B֐̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese `B֐̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese `B֐̃}CNzʒu
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese mۂvf(Freq)
 \param iReservedLocSize \~English It's the size of array
 \~Japanese mۂvf(Location)
 \param iReservedMicSize \~English It's the size of array
 \~Japanese mۂvf(Mic)
*/
void SMDatabaseAlloc_r(MICARY_SMDatabase * SMD, int iFreqSize, int iLocSize, int iMicSize,
											  int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize) ;
#endif

/**
 \brief \~English It frees database of separation matrix
 \brief \~Japanese si[zJ
 \param SMD \~English It's Database of separation matrix
 \~Japanese si[z܂ލ\
*/
void SMDatabaseFree(MICARY_SMDatabase * SMD) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees database of separation matrix
 \brief \~Japanese si[z̎gpNA
 \param SMD \~English It's Database of separation matrix
 \~Japanese si[z܂ލ\
*/
void SMDatabaseClear(MICARY_SMDatabase * SMD) ;
#endif

/**
 \brief \~English It initializes complex array
 \brief \~Japanese Complexz̏(0)
 \param array \~English It's complex array
 \~Japanese Complexz܂ލ\
*/

void ComplexArrayInit(MICARY_ComplexArray * array) ;

/**
The structure contains Update Separation matrix.
\author Sachiko Suzuki
\date 6 Feb 2009
*/
typedef struct{
	MICARY_IntArray micary_iaUpdate ;	///<\~Japanese XVtO
	MICARY_SMDatabase micary_dbSeparation ;	///<\~Japanese s
} MICARY_UpdateSeparationMat ;

/**
 \brief \~English It allocates update separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_UpdateSeparationMat\̂̊mہi1:IC0:G[j
 \param SPM \~English It's the update separation matrix struct pointer .
 \~Japanese XVps\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of source location.
 \~Japanese }CN
 */
void UpdateSeparationMatAlloc(MICARY_UpdateSeparationMat * SPM, int iFreqSize, int iLocSize,int iMicSize) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates update separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_UpdateSeparationMat\̂̊mہi1:IC0:G[j
 \param SPM \~English It's the update separation matrix struct pointer .
 \~Japanese XVps\̃|C^
 \param iFreqSize \~English It's the size of frequency.
 \~Japanese gC
 \param iLocSize \~English It's the number of source location.
 \~Japanese ʒu
 \param iMicSize \~English It's the number of source location.
 \~Japanese }CN
 */
void UpdateSeparationMatAlloc_r(MICARY_UpdateSeparationMat * SPM, int iFreqSize, int iLocSize,int iMicSize,
							  int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize) ;
#endif

/**
 \brief \~English It frees update separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_UpdateSeparationMat\̂Ji1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese XVps\̃|C^
*/
 void UpdateSeparationMatFree(MICARY_UpdateSeparationMat * SPM) ;
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees update separation matrix.(1:true, 0:false)
 \brief \~Japanese MICARY_UpdateSeparationMat\̂gpNAi1:IC0:G[j
 \param SPM \~English It's the separation matrix struct pointer .
 \~Japanese XVps\̃|C^
*/
void UpdateSeparationMatClear(MICARY_UpdateSeparationMat * SPM) ;
#endif

//
#ifdef __cplusplus
}
#endif
// -------------------------------------------------------------------------------------
#endif // MICARY_GHDSS_struct_h_defined

