/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */

#include <iostream>
#include "BufferedNode.h"
#include "Buffer.h"
#include "Vector.h"
#include <math.h>
#include <assert.h>
#include "Source.h"

using namespace std;
using namespace FD;

class ConstantLocalization;

DECLARE_NODE(ConstantLocalization);
/*Node
 *
 * @name ConstantLocalization
 * @category HARK:Localization
 * @description Output constant source locations.
 *
 * @output_name SOURCES
 * @output_type Vector<ObjectRef>
 * @output_description Source locations. The type of the element is "Source".
 *
 * @parameter_name ANGLES
 * @parameter_type object
 * @parameter_value <Vector<float> >
 * @parameter_description A vector to specify source locations. Each vector element shows azimuth of a source in degree.
 *
 * @parameter_name ELEVATIONS
 * @parameter_type object
 * @parameter_value <Vector<float> >
 * @parameter_description Elevations of source locations. Each element represents an elevation of each source location.
 *
 END*/

class ConstantLocalization : public BufferedNode {
	int sourcesID;
	Vector<float> angles;
	Vector<float> elevations;
    float distance;

public:
	ConstantLocalization(string nodeName, ParameterSet params)
		: BufferedNode(nodeName, params)
    {
        sourcesID = addOutput("SOURCES");

		angles = object_cast<Vector<float> >(parameters.get("ANGLES"));
		elevations = object_cast<Vector<float> >(parameters.get("ELEVATIONS"));

		assert(angles.size() == elevations.size());

		inOrder = true;
	}

    void calculate(int output_id, int count, Buffer &out)
    {
		RCPtr<Vector<ObjectRef> > psources(new Vector<ObjectRef>);
        Vector<ObjectRef>& sources = *psources;
        (*(outputs[sourcesID].buffer))[count] = psources;

		for (int i = 0; i < angles.size(); i++) {
            RCPtr<Source> src(new Source);
            
            src->power = 1.0f; //by shohei
            double theta = angles[i] * M_PI / 180.0;
            double phi = elevations[i] * M_PI / 180.0;
            double cos_phi = cos(phi);
            src->x[0] = (float)(cos(theta) * cos_phi);
            src->x[1] = (float)(sin(theta) * cos_phi);
            src->x[2] = (float)(sin(phi));
            src->id = i;
            
            psources->push_back(src);
		}
    }
};
