#include <gtk/gtk.h>
#include <string>

static GtkWidget* window = NULL;
static GtkWidget* drawingarea = NULL;
static GdkPixmap* pixmap = NULL;
static GdkRectangle plot_area;
static GdkGC* plot_gc[3];
static GdkGC* cursor_gc;
static int start_time = 0;
static int time_length = 2000;
static bool is_alive = true;


void init_source_location_window(std::string name);
gboolean delete_event(GtkWidget* widget, GdkEventConfigure* event);
gboolean configure_event(GtkWidget* widget, GdkEventConfigure* event);
gboolean expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer data);
void draw_plot_area();

int main(int argc, char* argv[])
{
	init_source_location_window("Source Location");

	while (gtk_main_iteration()) {
	}
	
	return 0;
}

void init_source_location_window(std::string name)
{
	gtk_set_locale();
	gtk_init(0, 0);

    is_alive = true;
    if (!window) {
      window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_window_set_title(GTK_WINDOW(window), name.c_str());
      gtk_widget_set_usize(window, 600, 300);
      
      drawingarea = gtk_drawing_area_new();
      gtk_widget_show(drawingarea);
      gtk_container_add(GTK_CONTAINER(window), drawingarea);
      
      gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                         (GtkSignalFunc)delete_event, NULL);
      gtk_signal_connect(GTK_OBJECT(drawingarea), "configure_event",
                         (GtkSignalFunc)configure_event, NULL);
      gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                         (GtkSignalFunc)expose_event, NULL);
      gtk_widget_show(window);
    }
}

bool source_location_window_is_valid()
{
  return window != NULL;
}

gboolean delete_event(GtkWidget* widget, GdkEventConfigure* event)
{
  /*
  GtkWidget* dialog;
  gint response;

  dialog = gtk_message_dialog_new(GTK_WINDOW(widget),
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_WARNING,
                                  GTK_BUTTONS_YES_NO,
                                  "Really quit?");

  response = gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);

  if (GTK_RESPONSE_YES == response) {
    window = NULL;
    is_alive = false;
    return FALSE;
  }
  */
  window = NULL;
  return FALSE;
  
  return TRUE;
}


gboolean configure_event(GtkWidget* widget, GdkEventConfigure* event)
{
  if (pixmap) {
    gdk_pixmap_unref(pixmap);
  }

  pixmap = gdk_pixmap_new(widget->window,
			  widget->allocation.width,
			  widget->allocation.height,
			  -1);

  GdkColor red = {0, 0xffff, 0x0000, 0x0000};
  GdkColor green = {0, 0x0000, 0xffff, 0x0000};
  GdkColor blue = {0, 0x0000, 0x0000, 0xffff};
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &red, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &green, FALSE, TRUE);
  gdk_colormap_alloc_color(gdk_colormap_get_system(), &blue, FALSE, TRUE);
  plot_gc[0] = gdk_gc_new(pixmap);
  plot_gc[1] = gdk_gc_new(pixmap);
  plot_gc[2] = gdk_gc_new(pixmap);
  gdk_gc_set_foreground(plot_gc[0], &red);
  gdk_gc_set_foreground(plot_gc[1], &green);
  gdk_gc_set_foreground(plot_gc[2], &blue);
  gdk_gc_set_line_attributes(plot_gc[0], 3, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
  gdk_gc_set_line_attributes(plot_gc[1], 3, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
  gdk_gc_set_line_attributes(plot_gc[2], 3, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);

  cursor_gc = gdk_gc_new(pixmap);
  gdk_gc_set_foreground(cursor_gc, &red);

  gdk_draw_rectangle(pixmap,
		     widget->style->white_gc,
		     TRUE, 0, 0,
		     widget->allocation.width,
		     widget->allocation.height);

  gdk_draw_rectangle(pixmap,
		     widget->style->black_gc,
		     FALSE, 0, 0,
		     widget->allocation.width,
		     widget->allocation.height);

  plot_area.x = widget->allocation.x + 30;
  plot_area.y = widget->allocation.y + 10;
  plot_area.width = widget->allocation.width - 40;
  plot_area.height = widget->allocation.height - 30;

  draw_plot_area();
  
  return TRUE;
}

gboolean expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer data)
{
#if 0
  gdk_window_clear_area(widget->window,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
  gdk_gc_set_clip_rectangle(widget->style->fg_gc[widget->state],
			    &event->area);
  //gdk_draw_arc(widget->window,
  //widget->style->fg_gc[widget->state],
  //TRUE,
  //0, 0, widget->allocation.width, widget->allocation.height,
  //0, 64 * 360);
  gdk_draw_rectangle(widget->window, widget->style->white_gc,
		     TRUE, 10, 10, 580, 280);
  gdk_gc_set_clip_rectangle(widget->style->fg_gc[widget->state], NULL);
#endif
  
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  return TRUE;
}

void draw_plot_area()
{
	if (!pixmap) {
		return;
	}

	//GdkFont* font = gdk_font_load("-*-courier-medium-r-normal-*-10-*-*-*-*-*-iso8859-*");
	GdkFont* font = gdk_font_load("fixed");

	gdk_draw_rectangle(pixmap,
					   drawingarea->style->white_gc,
					   TRUE, drawingarea->allocation.x, drawingarea->allocation.y,
					   drawingarea->allocation.width,
					   drawingarea->allocation.height);

	gdk_draw_rectangle(pixmap,
					   drawingarea->style->black_gc,
					   FALSE, plot_area.x, plot_area.y,
					   plot_area.width, plot_area.height);

	char label[32];
	for (int angle = -180; angle <= 180; angle += 30) {
		snprintf(label, sizeof(label), "% 4d", angle);
		gdk_draw_string(pixmap, font, drawingarea->style->black_gc,
						plot_area.x - 28,
						plot_area.y + plot_area.height * (180 - angle) / 360 + 5,
						label);
	}

	for (int time = 0; time <= time_length; time += 100) {
		snprintf(label, sizeof(label), "%d", start_time + time);
		gdk_draw_string(pixmap, font, drawingarea->style->black_gc,
						plot_area.x + plot_area.width * time / time_length,
						plot_area.y + plot_area.height + 10,
						label);
	}
}

void init_start_time()
{
  start_time = 0;
}

void init_time_length(int tl)
{
  time_length = tl;
}

void update_time(int cur_time)
{
	if (cur_time > start_time + time_length) {
		start_time += time_length / 2;
	}
}

int get_start_time() { return start_time; }
int get_time_length() { return time_length; }

int get_x_in_plot_area(int time)
{
	return plot_area.x + (time - start_time) * plot_area.width / time_length;
}

int get_y_in_plot_area(int angle)
{
	return plot_area.y + (180 - angle) * plot_area.height / 360;
}

void draw_lines(GdkPoint* points, int size, int id)
{
	gdk_draw_lines(pixmap, plot_gc[id % 3],
				   points, size);
}

void draw_cursor(int time)
{
	gdk_draw_line(pixmap, cursor_gc,
				  plot_area.x + (time - start_time) * plot_area.width / time_length,
				  plot_area.y,
				  plot_area.x + (time - start_time) * plot_area.width / time_length,
				  plot_area.y + plot_area.height);
}

void update_window()
{
	GdkRectangle update_rect;
	update_rect.x = drawingarea->allocation.x;
	update_rect.y = drawingarea->allocation.y;
	update_rect.width = drawingarea->allocation.width;
	update_rect.height = drawingarea->allocation.height;
	gtk_widget_draw(drawingarea, &update_rect);
}

bool get_state_alive()
{
  return is_alive;
}
